###########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import time
from zope.interface import implements, Interface
from twisted.internet import reactor
from twisted.python import log

from pyGridWare.wsrf.faults.PropertiesFaults import ResourceUnknownFault

from interfaces import IResourceHomeManager, IResourceHome, IPersistent
from ResourceSweeper import ResourceSweeper

                
class ResourceHomeManager:
    """Manage ResourceHome instances while acting as an 
    adapter for ResourceHome.
    
    Instance variables:
        context -- IResourceHome implementation.
    """
    implements(IResourceHomeManager, IPersistent)
    sweeper = ResourceSweeper()
    
    def __init__(self, klass):
        """Instance Attributes:
        __homes -- dict of ResourceHome instances
        __persistent --
        """
        self.__klass = klass
        self.__homes = {}
        self.__persistent = IPersistent.providedBy(klass)
        
        if not IResourceHome.implementedBy(klass):
            raise TypeError, 'klass paramter must implement IResource'
            
        if self.__persistent:
            self.load()
            reactor.addSystemEventTrigger('before', 'shutdown', self.save)
    
    def _getResourceHome(self, request=None):
        """return ResourceHome instance
        """
        if request is None or not hasattr(request, 'portal'):
            #TODO: cleanup ANONYMOUS
            key = getattr(self.__klass, 'ANONYMOUS', 0)
            home = self.getResourceHome(key)
            if home is None:
                home = self.newResourceHome()
                self.setResourceHome(key, home)
            log.msg("RESOURCEHOME: %s" %home, debug=True)
            log.msg("HOMES: %s" %self.__homes, debug=True)
            return home

        portal = request.portal
        credential = getattr(request, 'credential', None)
        d = portal.login(credential, None, IResourceHome)
        from twisted.python.failure import Failure
        if isinstance(d.result, Failure):
            d.result.raiseException()
        
        result = d.result
        log.msg("PORTAL RESOURCEHOME: %s" %result[1], debug=True)
        return result[1]
    
    # IResourceManagerHome
    def newResourceHome(self):
        return self.__klass()
        
    def setResourceHome(self, key, home):
        if self.__homes.has_key(key):
            raise KeyError, 'must use unique key'
        
        if not isinstance(home, self.__klass):
            raise TypeError, 'home must be instance of "%s"' %self.__klass
        
        #TODO: this isn't part of the interface
        #if self.__persistent:
        home.setAvatarId(key)
            
        self.sweeper.addResourceHome(home)
        self.__homes[key] = home
        
    def getResourceHome(self, key):
        return self.__homes.get(key)
        
    # IResourceHome
    def add(self, ctx, key=None, **kw):
        """save resource context referenced via key.
        """
        home = self._getResourceHome(**kw)
        #TODO: Cleanup home/manager naming
        ctx.ihome = home
        return home.add(ctx, key=key)
        
    def create(self, **kw):
        """create a new key.
        """
        home = self._getResourceHome(**kw)
        return home.create()
        
    def delete(self, key, **kw):
        """immediately delete resource context referenced via key.
        """
        home = self._getResourceHome(**kw)
        return home.delete(key)
        
    def get(self, key, **kw):
        """get resource context via key.
        """
        home = self._getResourceHome(**kw)
        return home.get(key)
        
    def setTerminationTime(self, key, terminationTime, **kw):
        """set terminationTime of resource, and schedule deletion of 
        resource context referenced via key.
        """
        home = self._getResourceHome(**kw)
        return home.setTerminationTime(key, terminationTime)
        
    def getResources(self, **kw):
        """return all resource contexts.
        """
        home = self._getResourceHome(**kw)
        return home.getResources()
        
    def getResourceContext(self, ps, address, **kw):
        """return resource context.
        """
        home = self._getResourceHome(**kw)
        return home.getResourceContext(ps, address)
        
    # IPersistent
    def save(self, key=None):
        """If key is None save all resources, else save only 
        resource specified via key.
        """
        if not self.__persistent: return
        for avatarId,home in self.__homes.items():
            self.__klass.save(home, key=key)
    
    def load(self):
        """Implement this method to load up state upon initialization.
        """
        if not self.__persistent: return
        log.msg('MANAGER LOAD', debug=True)
        if len(self.__homes) > 0:
            raise RuntimeError, 'ResourceHomeManager can not be initialized'
        
        log.msg('HOME LOAD', debug=True)
        for home in self.__klass.load():
            avatarId = home.getAvatarId()
            self.__homes[avatarId] = home
        
        
        
