############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
########################################################################### 
import base64
from M2Crypto import SSL
from M2Crypto import BIO
from pyGsi import proxylib
import pyGsi._mic


class M2SecurityContext:
    
    def __init__(self, sslctx, connect=True, delegation=False):
        """
        sslctx -- SSL Context
        delegation -- 
        """
        self._context = sslctx
        self._delegate = delegation
        
        self._connection = conn = SSL.Connection(sslctx)
        self._readbio = readbio = BIO.MemoryBuffer()
        self._writebio = writebio = BIO.MemoryBuffer()
        self._sslbio = sslbio = BIO.SSLBio()
        
        conn.set_bio(readbio, writebio)
        sslbio.set_ssl(conn)
        if connect:
            conn.set_connect_state()
        else:
            conn.set_accept_state()

    def initContext(self, inTokens=None):
        """client-side
        if delegation is set write 'D' into
        the sslbio, otherwise write 0 after handshake.
        Parameters:
            inTokens --

        return bool,outputTokens
        """
        handshake_complete = True
        if inTokens is not None:
            write_len,token_length = 0,len(inTokens)
            while write_len < token_length: 
                write_len += self._readbio.write(inTokens[write_len:])
        
        if self._sslbio.do_handshake() <= 0:
            if not self._sslbio.should_retry() or not self._sslbio.should_read():
                raise RuntimeError, "handshake incomplete, sslbio error state"

            handshake_complete = False
        elif self._delegate:
            self._sslbio.write("D")
        else:
            self._sslbio.write("0")

        return handshake_complete,self._writebio.read()
    
    def acceptContext(self, inTokens=None):
        """server-side
        Parameters:
            inTokens --

        return bool,outputTokens
        """
        handshake_complete = False
        self._readbio.write(inTokens)
        if self._sslbio.do_handshake() <= 0:
            if not self._sslbio.should_retry() or not self._sslbio.should_read():
                raise RuntimeError, "handshake incomplete, sslbio error state"
        else:
            handshake_complete = True
        
        return handshake_complete, self._writebio.read()

    def setDelegation(self, val):
        """
        Set the delegation member variable
        """
        self._delegate = val
       
    def getMIC(self, signedInfoC14n):
        """
        Create a message integrity code from 
        the cannonicalized message
        """
        ssl = self._connection.ssl
        return pyGsi._mic.ssl_generate_mic(ssl, signedInfoC14n)

    def verifyMIC(self, signedInfoC14n, signatureValue):
        """
        Verify a the message integrity code from the
        cannonicalized message
        """
        ssl = self._connection.ssl
        return pyGsi._mic.ssl_verify_mic(ssl, signedInfoC14n, signatureValue)


class GSIAdapter:
    """
    Associates a GSSContext with a context Id. It is used 
    for GSI secure converstation.
    """

    def __init__(self, sslctx, **kw):
        self._context = M2SecurityContext(sslctx, **kw)
        self._contextID = None
        self._done = False

    def getContext(self):
        """org.ietf.jgss.GSSContext getContext()
        """
        return self._context

    def getContextID(self):
        """java.lang.String getContextId()
        """
        return self._contextID

    def setContext(self, context):
        self._context = context

    def setContextID(self, contextID):
        self._contextID = contextID

    def contextCreated(self):
        return self._done
    
    def acceptSecurityContext(self, inToken64=None):
        """
        **Arguments**

        - *inToken* base64 encoded tokens, which contain
        authentication information
        
        **Returns**
        a string of base64 encoded tokens that has no newlines
        """
        tokens = None
        if inToken64 is not None:
            tokens = base64.decodestring(inToken64)

        self._done, outToken = self._context.acceptContext(tokens)
        if outToken is not None:
            tokens64 = base64.encodestring(outToken)
            outTokenSplit = tokens64.split('\n')
            outTokens = ""

            for token in outTokenSplit:
                outTokens += token
            return outTokens

        return None
        
    def initSecurityContext(self, inToken64=None):
        """
        **Arguments**
        - *inToken* - must be base64 encoded
        
        **Returns**
        a string of base64 encoded tokens that has no newlines
        """
        inToken = None
        if inToken64 is not None:
            inToken = base64.decodestring(inToken64)
            
        handshake_complete, outToken = self._context.initContext(inToken)
        tokens64 = base64.encodestring(outToken)
        outTokenSplit = tokens64.split('\n')
        outTokens = ""
        for token in outTokenSplit:
            outTokens += token

        return outTokens
     
    def getMIC(self, signedInfoC14n):
        """
        Create a message integrity code from 
        the cannonicalized message
        """
        return self._context.getMIC(signedInfoC14n)

    def verifyMIC(self, signedInfoC14n, signatureValue):
        """
        Verify a the message integrity code from the
        cannonicalized message
        """
        return self._context.verifyMIC(signedInfoC14n, signatureValue)
