#!/usr/bin/env python
##########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import optparse, os, sys, signal, time, unittest, subprocess
from urlparse import urlunparse
from os.path import isfile, join, abspath, sep
from ConfigParser import ConfigParser
from twisted.internet import reactor
from twisted.python import log

from ZSI.wstools.Namespaces import WSA200403
from pyGridWare.utility.generate import wsdl2web
import pyGridWare.wsrf.faults

try:
    from pyGridWare.utility import log as _log
    if int(os.environ.get('NETLOGGER_ON')):
        log.addObserver(_log.NetLoggerObserver(os.environ.get('NETLOGGER_DEST'))._emit)
except:
    _log = None

sys.path.append('suites')
import suites

PID = None

_path = lambda unix_path: abspath(join(*unix_path.split('/')))
_listdir = lambda dir: map(lambda f: join(dir,f), os.listdir(dir))

class MyTestSuite(unittest.TestSuite):
    def __init__(self, tests=()):
        unittest.TestSuite.__init__(self, tests=tests)

    def run(self, result, **kw):
        """Pass key word dict down to suites/tests
        """
        for test in self._tests:
            if result.shouldStop: 
                break
            test(result, **kw) 
        return result
    __call__ = run
unittest.TestLoader.suiteClass = MyTestSuite


class MyTextTestRunner(unittest.TextTestRunner):
    """A test runner class that displays results in textual form.

    It prints out the names of tests as they are run, errors as they
    occur, and a summary of the results at the end of the test run.
    """
    def run(self, test, **kw):
        """Run the given test case or test suite.
        """
        result = self._makeResult()
        startTime = time.time()
        test(result, **kw)
        stopTime = time.time()
        timeTaken = stopTime - startTime
        result.printErrors()
        self.stream.writeln(result.separator2)
        run = result.testsRun
        self.stream.writeln("Ran %d test%s in %.3fs" %
                            (run, run != 1 and "s" or "", timeTaken))
        self.stream.writeln()
        if not result.wasSuccessful():
            self.stream.write("FAILED (")
            failed, errored = map(len, (result.failures, result.errors))
            if failed:
                self.stream.write("failures=%d" % failed)
            if errored:
                if failed: self.stream.write(", ")
                self.stream.write("errors=%d" % errored)
            self.stream.writeln(")")
        else:
            self.stream.writeln("OK")
        return result




"""MyTestProgram: 
Expects the module containing each test case to define two globals.  If
you do not, createTests will break (AttributeError).
    WSDL -- path to WSDL file.
    TOP -- top level name for generated modules/WSDL
    SECTION -- name of generated stub module.
 
"""
class MyTestProgram(unittest.TestProgram):
    USAGE = """\
Usage: %(progName)s [options] [test] [...]

Options:
  -h, --help       Show this message
  -w, --off        Turn off wsdl to python generation.
  -v, --verbose    Verbose output
  -q, --quiet      Minimal output
  -s, --ssl        Use SSL (must have M2Crypto installed)
  -g, --gsi        Use GSI (must have pyGsi and M2Crypto installed)
  -i, --ds         Use secure conversation & digital signatures

Security:
   ssl -- tests a vanilla SSL setup, certificates must be created in
       the "etc/certificates" directory.  See the README.

   gsi -- Grid Security Infrastructure, this option requires a typical
       "/etc/grid-security/" setup and a valid rfc 3820 compliant user 
       proxy certificate.

   ds -- 

Examples:
  %(progName)s                               - run default set of tests
  %(progName)s MyTestSuite                   - run suite 'MyTestSuite'
  %(progName)s MyTestCase.testSomething      - run MyTestCase.testSomething
  %(progName)s MyTestCase                    - run all 'test*' test methods
                                               in MyTestCase
"""

    def __init__(self, module='__main__', defaultTest=None, argv=None, 
                 testRunner=None, testLoader=unittest.defaultTestLoader):
        self.generate = True
        if type(module) == type(''):
            self.module = __import__(module)
            for part in module.split('.')[1:]:
                self.module = getattr(self.module, part)
        else:
            self.module = module
        if argv is None:
            argv = sys.argv
            
        self.ssl = self.gsi = self.ds = None
        self.verbosity = 1
        self.defaultTest = defaultTest
        self.testRunner = testRunner
        self.testLoader = testLoader
        self.progName = os.path.basename(argv[0])
        if self.parseArgs(argv):
            self.runTests()

    def usageExit(self, msg=None):
        if msg: print msg
        print self.USAGE % self.__dict__
        reactor.callWhenRunning(reactor.stop)

    def parseArgs(self, argv):
        import getopt
        global PID
        self.scheme = 'http'
        self.path = ''
        
        try:
            options, args = getopt.getopt(argv[1:], 'hHvwqsgi',
                              ['help','verbose','off','quiet', 'ssl', 'gsi', 'ds'])
            for opt, value in options:
                if opt in ('-h','-H','--help'):
                    self.usageExit()
                    return False
                if opt in ('-w','--off'):
                    self.generate = False
                if opt in ('-q','--quiet'):
                    self.verbosity = 0
                if opt in ('-v','--verbose'):
                    self.verbosity = 2
                if opt in ('-s', '--ssl'):
                    from M2Crypto import SSL
                    self.ssl = True
                    self.scheme = 'https'
                if opt in ('-g', '--gsi'):
                    import pyGsi
                    if self.ssl: 
                        self.usageExit('ERROR: ssl and gsi options are mutually exclusive')
                        return False

                    self.gsi = True
                    self.scheme = 'https'
                if opt in ('-c', '--secureconv'):
                    self.ds = True

                    
            self.client_kw = kw = {}
            kw['wsAddressURI'] = WSA200403.ADDRESS
            
            if len(args) == 0 and self.defaultTest is None:
                self.test = self.testLoader.loadTestsFromModule(self.module)
                return
            if len(args) > 0:
                self.testNames = args
            else:
                self.testNames = (self.defaultTest,)
            self.createTests()
        except getopt.error, msg:
            self.usageExit(msg)
            return False
        except RuntimeError, ex:
            log.err(ex)
            self.usageExit(msg)
            return False

        tac = 'server-config.tac'
        slog = 'server.log'
        if self.ssl:
            from M2Crypto import SSL
            from pyGridWare.utility.m2crypto import ClientContextFactory

            cert,key,calist = map(_path, ('etc/certificates/client.pem', 
                  'etc/certificates/client.key','etc/certificates/ca.pem'))

            if not os.path.isfile(cert):
                raise RuntimeError, 'Missing certificate %s' %cert
            if not os.path.isfile(key):
                raise RuntimeError, 'Missing key %s' %key

            kw['contextFactory'] = ClientContextFactory(key, cert, calist=calist, 
                                       sslmethod=SSL.SSLv3_METHOD,)
            tac = 'm2crypto-server-config.tac'
            slog = 'm2crypto-server.log'

        elif self.gsi:
            from M2Crypto import SSL
            import pyGridWare.utility.m2crypto
            from pyGridWare.utility.gsi import GSIClientContextFactory
            kw['contextFactory'] = GSIClientContextFactory()
            tac = 'gsi-server-config.tac'
            slog = 'gsi-server.log'

        elif self.ds:
            from M2Crypto import SSL
            import pyGridWare.utility.m2crypto
            from pyGridWare.utility.gsi import GSIClientContextFactory
            kw['contextFactory'] = GSIClientContextFactory()
            slog = 'ds-server.log'
 
        # Data Files Directory
        tac = join("etc", "tac", tac)
        arg = "twistd -noy %s -l %s" %(tac, slog)
        p = subprocess.Popen(arg, shell=True)
        PID = p.pid
        print "wait for server '%s' process to initialize..." %tac
        time.sleep(5)
        return True

    def _readServerConfig(self):
        '''read server configuration file to set up URL.
        '''
        cp = ConfigParser()
        cp.read('server-config.txt')
        netloc = '%s:%d' %(cp.get('Location', 'host'), cp.getint('Location', 'port'))
        self.client_kw['url'] = urlunparse((self.scheme, netloc, '', '', '', ''))
     
    def createTests(self):
        # -v
        if self.verbosity > 1:
            log.startLogging(sys.stdout)
            if _log is not None: _log.addNetLoggerObserver()
 
        try:
            unittest.TestProgram.createTests(self)
        except AttributeError, ex:
            self.usageExit(ex)

        cp = ConfigParser()
        if self.generate is False:
            self._readServerConfig()
            return
        
        tests = []
        def flatten(i): 
            if isinstance(i, unittest.TestCase): tests.append(i)
            if isinstance(i, unittest.TestSuite): 
                    for j in i._tests: flatten(j)
        flatten(self.test)
 
        modset = set(map(lambda t:eval(t.__module__), tests))
        BigFunk = lambda f,w: ' %s' %os.path.realpath(join(os.path.dirname(f),w))

        failed = []
        for wsdl,top in list(set(map(lambda mod: (BigFunk(mod.__file__,mod.WSDL),getattr(mod,'TOP',None)), modset))):
            wsdl2web.run(wsdl, 
                force=True, 
                top=top, 
                config=os.path.join('server-config.txt'),
                tac=os.path.join('etc', 'tac', 'server-config.tac'),
            )

        self._readServerConfig()
      
    def runTests(self):
        log.msg("RUN TESTS")
        if self.testRunner is None:
            self.testRunner = MyTextTestRunner(verbosity=self.verbosity)
        result = self.testRunner.run(self.test, **self.client_kw)
        log.msg("SUCCESS?: ", result, debug=True)
        log.msg("SUCCESS?: ", result.wasSuccessful(), debug=True)

        if True:
            reactor.callLater(0.1, reactor.stop)
            reactor.callLater(0.1, os.kill, PID, signal.SIGTERM)



def main():
    reactor.callWhenRunning(MyTestProgram, 
                            defaultTest="suites.counter.WSRFCounterTestSuite")
    reactor.run(installSignalHandlers=0)


if __name__ == '__main__': 
    main()
