#!/usr/bin/env python
##########################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
import unittest, os
from ZSI import ZSIException
import counter
from os.path import abspath, join, sep
_resolvepath = lambda f: abspath(join(*f.split('/')))
WSDL = counter.WSDL


class WrappedCounterTestCase(counter.CounterTestCase):
    """Wrap a counter test case for several purposes.  
    """
    path = '/wsrf/services/PortalCounterService'
    #path = '/wsrf/services/CounterService'
    
    def __init__(self, klass, key=None, cert=None, calist=None, **kw):
        """Instance Attributes:
        key --
        cert --
        calist -- 
        """
        self.key = key
        self.cert = cert
        self.calist = calist or []
        unittest.TestCase.__init__(self, **kw)
        self.wrappedTest = klass(**kw)

    def getEPR(self):
        return self.wrappedTest.getEPR()

    def setEPR(self, epr):
        self.wrappedTest.setEPR(epr)

    def NewRequestInstance(self):
        raise NotImplementedError, 'this method is not implemented'
        
    def setUp(self):
        kw = self.client_kw
        #if kw['contextFactory'].__class__.__module__ != 'pyGridWare.utility.ssl':
        #    raise RuntimeError, 'expecting pyOpenSSL for client, got: %s' %kw['contextFactory']
        
        #from twisted.internet.ssl import SSL
        #from pyGridWare.utility.ssl import ClientContextFactory
        from pyGridWare.utility.m2crypto import ClientContextFactory, SSL

        key = self.key
        cert = self.cert
        calist = self.calist
        # replacement contextFactory? 
        if not (key is None and cert is None):
            f = filter(lambda f: not os.path.isfile(f), [key, cert]+calist)
            if len(f): 
                raise RuntimeError, 'missing files %s' %str(f)
    
            kw['contextFactory'] =\
                ClientContextFactory(key, cert, sslmethod=SSL.SSLv3_METHOD, 
                                     calist=calist)
                
        self.wrappedTest.client_kw = kw
        self.wrappedTest.setUp()
        
    def tearDown(self):
        self.shouldStop = True

    def runTest(self):
        self.wrappedTest.runTest()

       
 
class AccessErrorTestCase(WrappedCounterTestCase):
    """Expects the test case to raise a ZSIException.
    """
    def runTest(self):
        self.failUnlessRaises(ZSIException, self.wrappedTest.runTest)
        


class SecureCounterTestCase(counter.CounterTestCase):
    """Wrap a counter test case for several purposes.  
    """
    path = '/wsrf/services/SecureCounterService'

    def __init__(self, klass, **kw):
        unittest.TestCase.__init__(self, **kw)
        self.wrappedTest = klass(**kw)

    def setUp(self):
        self.wrappedTest.client_kw = self.client_kw
        self.wrappedTest.setUp()


class SecureConversationCounterTestSuite(unittest.TestSuite):
    """
    """
    def __init__(self, tests=()):
        unittest.TestSuite.__init__(self, tests=tests)

    def run(self, result, **kw):
        """Factory method first.  Each factory method call will change the EPR
        for subsequent calls.
        """
        from pyGridWare.utility.gsi import GSIClientContextFactory
        from pyGridWare.security.m2.M2SignatureHandler import M2SignatureHandler
        from pyGridWare.security.m2.Authenticate import SecureConversationClient

        factory = GSIClientContextFactory()
        sc = SecureConversationClient()
        ctx_id = sc.exchange(factory.getContext(), wsAddressURI=kw['wsAddressURI'], 
                    url='%s-authService' %kw['url'])

        kw['sig_handler'] = handler = M2SignatureHandler()
        handler.setSecureConversation(ctx_id)

        epr = None
        for test in self._tests:
            if result.shouldStop: 
                break

            if isinstance(test, CounterTestCase):
                # counter testcase
                test.setEPR(epr)
                test(result, **kw)
                epr = test.getEPR()

                if isinstance(test, SubscribeMarker):
                    # consumer only retained w/i a TestSuite
                    kw['consumer'] = test.getConsumerService()
            else:
                # counter suite
                if kw.has_key('consumer'): 
                    kw['consumer'].stop()
                    del kw['consumer']
                test(result, **kw)
                
        return result
    __call__ = run


def BasicServiceInstanceTestSuite():
    """Create new SSL Context in order to use a different credential,
    then try to access a service resource instance created with the
    original credential and protected by ResourcePortal.

    Receives a "ResourceUnknownFault", because we can not see the
    the resource as a different user.
    """
    kw1 = {}
    kw1['key'] = _resolvepath('etc/certificates/client2.key')
    kw1['cert'] =  _resolvepath('etc/certificates/client2.pem')
    kw1['calist'] = map(_resolvepath, ['etc/certificates/ca.pem',])

    return \
        counter.CounterTestSuite(\
            tests=(WrappedCounterTestCase(counter.CreateTestCase), 
                   WrappedCounterTestCase(counter.AddTestCase), 
                   AccessErrorTestCase(counter.AddTestCase, **kw1),
                   WrappedCounterTestCase(counter.DestroyTestCase),
            )
        )


def ServiceInstanceTestSuite():
    """Create new SSL Context in order to use a different credential,
    then try to access a service resource instance created with the
    original credential and protected by ResourcePortal.

    Receives a "ResourceUnknownFault", because we can not see the
    the resource as a different user.
    """
    kw1 = {}
    kw1['key'] = _resolvepath('etc/certificates/client2.key')
    kw1['cert'] =  _resolvepath('etc/certificates/client2.pem')
    kw1['calist'] = map(_resolvepath, ['etc/certificates/ca.pem',])

    return \
        counter.CounterTestSuite(\
            tests=(WrappedCounterTestCase(counter.CreateTestCase), 
                   WrappedCounterTestCase(counter.AddTestCase), 
                   AccessErrorTestCase(counter.AddTestCase, **kw1),
                   WrappedCounterTestCase(counter.CreateTestCase, **kw1),
                   WrappedCounterTestCase(counter.AddTestCase, **kw1),
                   AccessErrorTestCase(counter.AddTestCase),
                   WrappedCounterTestCase(counter.DestroyTestCase, **kw1),
            )
        )
        
        
def GSIAuthzTestSuite():
    """Create new SSL Context in order to use a different credential,
    then try to access a service resource instance created with the
    original credential and protected by ResourcePortal.

    Receives a "ResourceUnknownFault", because we can not see the
    the resource as a different user.
    """
    kw1 = {}
    kw1['key'] = _resolvepath('etc/certificates/client2.key')
    kw1['cert'] =  _resolvepath('etc/certificates/client2.pem')
    kw1['calist'] = map(_resolvepath, ['etc/certificates/ca.pem',])

    # 
    # this will get a authentication error, need to
    # use another cred that has a gridmap entry
    # 
    tests=(WrappedCounterTestCase(counter.CreateTestCase), 
               WrappedCounterTestCase(counter.AddTestCase), 
               AccessErrorTestCase(counter.AddTestCase, **kw1),
               WrappedCounterTestCase(counter.DestroyTestCase),)

    for t in tests:
        t.path = '/wsrf/services/GSIPortalCounterService'

    return counter.CounterTestSuite(tests=tests)
        
       
def SecureCounterTestSuite():
    tests = (SecureCounterTestCase(counter.CreateTestCase), SecureCounterTestCase(counter.AddTestCase), 
               SecureCounterTestCase(counter.DestroyTestCase))

    return SecureConversationCounterTestSuite(tests)
